@@ -6,7 +6,7 @@ import numbers
 
 from .healSparseCoverage import HealSparseCoverage
 from .utils import reduce_array, check_sentinel, _get_field_and_bitval, WIDE_NBIT, WIDE_MASK
-from .utils import is_integer_value
+from .utils import is_integer_value, _compute_bitshift
 from .fits_shim import HealSparseFits, _make_header, _write_filename
 
 
@@ -17,7 +17,7 @@ class HealSparseMap(object):
 
     def __init__(self, cov_map=None, cov_index_map=None, sparse_map=None, nside_sparse=None,
                  healpix_map=None, nside_coverage=None, primary=None, sentinel=None,
-                 nest=True, metadata=None):
+                 nest=True, metadata=None, _is_view=False):
         """
         Instantiate a HealSparseMap.
 
@@ -48,6 +48,9 @@ class HealSparseMap(object):
            If input healpix map is in nest format.  Default is True.
         metadata : `dict`-like, optional
            Map metadata that can be stored in FITS header format.
+        _is_view : `bool`, optional
+           This healSparse map is a view into another healsparse map.
+           Not all features will be available.  (Internal usage)
 
         Returns
         -------
@@ -83,6 +86,7 @@ class HealSparseMap(object):
         self._wide_mask_width = 0
         self._primary = primary
         self.metadata = metadata
+        self._is_view = _is_view
         if self._sparse_map.dtype.fields is not None:
             self._is_rec_array = True
             if self._primary is None:
@@ -166,7 +170,7 @@ class HealSparseMap(object):
 
     @classmethod
     def make_empty(cls, nside_coverage, nside_sparse, dtype, primary=None, sentinel=None,
-                   wide_mask_maxbits=None, metadata=None):
+                   wide_mask_maxbits=None, metadata=None, cov_pixels=None):
         """
         Make an empty map with nothing in it.
 
@@ -187,48 +191,61 @@ class HealSparseMap(object):
            Create a "wide bit mask" map, with this many bits.
         metadata : `dict`-like, optional
            Map metadata that can be stored in FITS header format.
+        cov_pixels : `np.ndarray` or `list`
+           List of integer coverage pixels to pre-allocate
 
         Returns
         -------
         healSparseMap : `HealSparseMap`
            HealSparseMap filled with sentinel values.
         """
+        test_arr = np.zeros(1, dtype=dtype)
 
         if wide_mask_maxbits is not None:
-            test = np.zeros(1, dtype=dtype)
-            if test.dtype != WIDE_MASK:
+            if test_arr.dtype != WIDE_MASK:
                 raise ValueError("Must use dtype=healsparse.WIDE_MASK to use a wide_mask")
             if sentinel is not None:
                 if sentinel != 0:
                     raise ValueError("Sentinel must be 0 for wide_mask")
             nbitfields = (wide_mask_maxbits - 1) // WIDE_NBIT + 1
 
-        cov_map = HealSparseCoverage.make_empty(nside_coverage, nside_sparse)
+        if cov_pixels is None:
+            cov_map = HealSparseCoverage.make_empty(nside_coverage, nside_sparse)
+            # One pixel is the overflow pixel of a truly empty map
+            npix = 1
+        else:
+            cov_pixels = np.atleast_1d(cov_pixels)
+            cov_map = HealSparseCoverage.make_from_pixels(nside_coverage, nside_sparse,
+                                                          cov_pixels)
+            # We need to allocate the overflow pixel
+            npix = cov_pixels.size + 1
 
         if wide_mask_maxbits is not None:
-            sparse_map = np.zeros((cov_map.nfine_per_cov, nbitfields), dtype=dtype)
+            # The sentinel is always zero
+            _sentinel = 0
+            sparse_map = np.zeros((cov_map.nfine_per_cov*npix, nbitfields), dtype=dtype)
+        elif test_arr.dtype.fields is None:
+            # Non-recarray
+            _sentinel = check_sentinel(test_arr.dtype.type, sentinel)
+            sparse_map = np.full(cov_map.nfine_per_cov*npix, _sentinel, dtype=dtype)
         else:
-            sparse_map = np.zeros(cov_map.nfine_per_cov, dtype=dtype)
-        if sparse_map.dtype.fields is not None:
+            # Recarray type
             if primary is None:
                 raise RuntimeError("Must specify 'primary' field when using a recarray for the sparse_map.")
 
             primary_found = False
-            for name in sparse_map.dtype.names:
+            for name in test_arr.dtype.names:
                 if name == primary:
-                    _sentinel = check_sentinel(sparse_map[name].dtype.type, sentinel)
-                    sparse_map[name][:] = _sentinel
+                    _sentinel = check_sentinel(test_arr[name].dtype.type, sentinel)
+                    test_arr[name] = _sentinel
                     primary_found = True
                 else:
-                    sparse_map[name][:] = check_sentinel(sparse_map[name].dtype.type, None)
+                    test_arr[name] = check_sentinel(test_arr[name].dtype.type, None)
 
             if not primary_found:
                 raise RuntimeError("Primary field not found in input dtype of recarray.")
 
-        else:
-            # fill with sentinel value
-            _sentinel = check_sentinel(sparse_map.dtype.type, sentinel)
-            sparse_map[:] = _sentinel
+            sparse_map = np.full(cov_map.nfine_per_cov*npix, test_arr, dtype=dtype)
 
         return cls(cov_map=cov_map, sparse_map=sparse_map,
                    nside_sparse=nside_sparse, primary=primary, sentinel=_sentinel,
@@ -236,7 +253,8 @@ class HealSparseMap(object):
 
     @classmethod
     def make_empty_like(cls, sparsemap, nside_coverage=None, nside_sparse=None, dtype=None,
-                        primary=None, sentinel=None, wide_mask_maxbits=None, metadata=None):
+                        primary=None, sentinel=None, wide_mask_maxbits=None, metadata=None,
+                        cov_pixels=None):
         """
         Make an empty map with the same parameters as an existing map.
 
@@ -258,6 +276,8 @@ class HealSparseMap(object):
            Create a "wide bit mask" map, with this many bits.
         metadata : `dict`-like, optional
            Map metadata that can be stored in FITS header format.
+        cov_pixels : `np.ndarray` or `list`
+           List of integer coverage pixels to pre-allocate
 
         Returns
         -------
@@ -282,7 +302,7 @@ class HealSparseMap(object):
 
         return cls.make_empty(nside_coverage, nside_sparse, dtype, primary=primary,
                               sentinel=sentinel, wide_mask_maxbits=wide_mask_maxbits,
-                              metadata=metadata)
+                              metadata=metadata, cov_pixels=cov_pixels)
 
     @staticmethod
     def _read_healsparse_file(filename, pixels=None):
@@ -300,7 +320,7 @@ class HealSparseMap(object):
         -------
         cov_map : `HealSparseCoverage`
            Coverage map with index values
-        sparse_map : `np.array`
+        sparse_map : `np.ndarray`
            Sparse map with map dtype
         nside_sparse : `int`
            Nside of the coverage map
@@ -394,7 +414,7 @@ class HealSparseMap(object):
 
         Parameters
         ----------
-        healpix_map : `np.array`
+        healpix_map : `np.ndarray`
            Numpy array that describes a healpix map.
         nside_coverage : `int`
            Nside for the coverage map to construct
@@ -408,7 +428,7 @@ class HealSparseMap(object):
         -------
         cov_map : `HealSparseCoverage`
            Coverage map with pixel indices
-        sparse_map : `np.array`
+        sparse_map : `np.ndarray`
            Sparse map of input values.
         """
         if not nest:
@@ -428,8 +448,8 @@ class HealSparseMap(object):
 
         cov_map.initialize_pixels(cov_pix)
 
-        sparse_map = np.zeros((cov_pix.size + 1)*cov_map.nfine_per_cov,
-                              dtype=healpix_map.dtype) + sentinel
+        sparse_map = np.full((cov_pix.size + 1)*cov_map.nfine_per_cov,
+                             sentinel, dtype=healpix_map.dtype)
         sparse_map[ipnest + cov_map[ipnest_cov]] = healpix_map[ipnest]
 
         return cov_map, sparse_map
@@ -463,11 +483,37 @@ class HealSparseMap(object):
         if self._is_wide_mask:
             s_hdr['WIDEMASK'] = self._is_wide_mask
             s_hdr['WWIDTH'] = self._wide_mask_width
-            _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map.flatten())
+            _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map.ravel())
         else:
             _write_filename(filename, c_hdr, s_hdr, self._cov_map[:], self._sparse_map)
 
-    def update_values_pix(self, pixels, values, nest=True):
+    def _reserve_cov_pix(self, new_cov_pix):
+        """
+        Reserve new coverage pixels.  This routine does no checking, it should
+        be done by the caller.
+
+        Parameters
+        ----------
+        new_cov_pix : `np.ndarray`
+           Integer array of new coverage pixels
+        """
+
+        new_cov_map = self._cov_map.append_pixels(len(self._sparse_map), new_cov_pix, check=False)
+        self._cov_map = new_cov_map
+
+        # Use resizing
+        oldsize = len(self._sparse_map)
+        newsize = oldsize + new_cov_pix.size*self._cov_map.nfine_per_cov
+
+        if self._is_wide_mask:
+            self._sparse_map.resize((newsize, self._wide_mask_width), refcheck=False)
+        else:
+            self._sparse_map.resize(newsize, refcheck=False)
+
+        # Fill with blank values
+        self._sparse_map[oldsize:] = self._sparse_map[0]
+
+    def update_values_pix(self, pixels, values, nest=True, operation='replace'):
         """
         Update the values in the sparsemap for a list of pixels.
 
@@ -477,89 +523,127 @@ class HealSparseMap(object):
            Integer array of sparse_map pixel values
         values : `np.ndarray`
            Value or Array of values.  Must be same type as sparse_map
+        operation : `str`, optional
+           Operation to use to update values.  May be 'replace' (default),
+           'or', or 'and' (for bit masks)
         """
 
+        if operation != 'replace':
+            if operation != 'or' and operation != 'and':
+                raise ValueError("Only replace, or, and and are supported operations")
+            if not self.is_integer_map or self._sentinel != 0:
+                raise ValueError("Can only use and/or with integer map with 0 sentinel")
+
+        # If _not_ recarray, we can use a single int/float
+        is_single_value = False
+        _values = values
+        if not self._is_rec_array:
+            if self._is_wide_mask:
+                # Special for wide_mask
+                if not isinstance(values, np.ndarray):
+                    raise ValueError("Wide mask must be set with a numpy ndarray")
+                if len(values) == self._wide_mask_width:
+                    is_single_value = True
+                    # Reshape so we can use the 0th entry below
+                    _values = _values.reshape((1, self._wide_mask_width))
+            else:
+                # Non wide_mask
+                if isinstance(values, numbers.Integral):
+                    if not self.is_integer_map:
+                        raise ValueError("Cannot set non-integer map with an integer")
+                    is_single_value = True
+                    _values = np.array([values], dtype=self.dtype)
+                elif isinstance(values, numbers.Real):
+                    if self.is_integer_map:
+                        raise ValueError("Cannot set non-floating point map with a floating point.")
+                    is_single_value = True
+                    _values = np.array([values], dtype=self.dtype)
+        if isinstance(values, np.ndarray) and len(values) == 1:
+            is_single_value = True
+
         # First, check if these are the same type
-        if not isinstance(values, np.ndarray):
-            raise RuntimeError("Values are not a numpy ndarray")
+        if not is_single_value and not isinstance(_values, np.ndarray):
+            raise ValueError("Values are not a numpy ndarray")
 
         if not nest:
             _pix = hp.ring2nest(self._nside_sparse, pixels)
         else:
             _pix = pixels
 
-        if self._sparse_map.dtype.type != values.dtype.type:
-            raise RuntimeError("Data-type mismatch between sparse_map and values")
+        # Check numpy data type for everything but wide_mask single value
+        if not self._is_wide_mask or (self._is_wide_mask and not is_single_value):
+            if self._is_rec_array:
+                if self._sparse_map.dtype != _values.dtype:
+                    raise ValueError("Data-type mismatch between sparse_map and values")
+            elif self._sparse_map.dtype.type != _values.dtype.type:
+                raise ValueError("Data-type mismatch between sparse_map and values")
 
-        if self._is_wide_mask:
-            if len(values.shape) != 2:
-                raise RuntimeError("Values must be WideBits or equivalent array.")
-            if values.shape[1] != self._wide_mask_width:
-                raise RuntimeError("Values must be WideBits or equivalent array with matched width.")
+        # Check array lengths
+        if not is_single_value and len(_values) != pixels.size:
+            raise ValueError("Length of values must be same length as pixels (or length 1)")
 
-        if len(values) == 1:
-            single_value = True
-        else:
-            single_value = False
-            if len(values) != pixels.size:
-                raise RuntimeError("Length of values must be the same as pixels, or length 1.")
+        if self._is_view:
+            # Check that we are not setting new pixels
+            if np.any(self.get_values_pix(_pix) <= self._sentinel):
+                raise RuntimeError("This API cannot be used to set new pixels in the map.")
 
         # Compute the coverage pixels
         ipnest_cov = self._cov_map.cov_pixels(_pix)
 
         # Check which pixels are in the coverage map
         cov_mask = self.coverage_mask
-        in_cov, = np.where(cov_mask[ipnest_cov])
-        out_cov, = np.where(~cov_mask[ipnest_cov])
+        in_cov = cov_mask[ipnest_cov]
+        out_cov = ~cov_mask[ipnest_cov]
 
         # Replace values for those pixels in the coverage map
-        if single_value:
-            self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] = values[0]
+        if is_single_value:
+            if operation == 'replace':
+                self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] = _values[0]
+            elif operation == 'or':
+                self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] |= _values[0]
+            elif operation == 'and':
+                self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] &= _values[0]
         else:
-            self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] = values[in_cov]
+            if operation == 'replace':
+                self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] = _values[in_cov]
+            elif operation == 'or':
+                self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] |= _values[in_cov]
+            elif operation == 'and':
+                self._sparse_map[_pix[in_cov] + self._cov_map[ipnest_cov[in_cov]]] &= _values[in_cov]
 
         # Update the coverage map for the rest of the pixels (if necessary)
-        if out_cov.size > 0:
-            # This requires data copying. (Even numpy appending does)
-            # I don't want to overthink this and prematurely optimize, but
-            # I want it to be able to work when the map isn't being held
-            # in memory.  So that will require an append and non-contiguous
-            # pixels, which I *think* should be fine.
+        if out_cov.sum() > 0:
+            # New version to minimize data copying
 
-            new_cov_pix = np.unique(ipnest_cov[out_cov])
-            if self._is_wide_mask:
-                sparse_append = np.zeros((new_cov_pix.size*self._cov_map.nfine_per_cov,
-                                          self._wide_mask_width),
-                                         dtype=self._sparse_map.dtype)
+            # Faster trick for getting unique values
+            new_cov_temp = np.zeros(cov_mask.size, dtype=np.int8)
+            new_cov_temp[ipnest_cov[out_cov]] = 1
+            new_cov_pix, = np.where(new_cov_temp > 0)
+
+            # Reserve the memory here
+            oldsize = len(self._sparse_map)
+            self._reserve_cov_pix(new_cov_pix)
+
+            if is_single_value:
+                if operation == 'replace':
+                    self._sparse_map[oldsize:][_pix[out_cov] + self._cov_map[ipnest_cov[out_cov]] -
+                                               oldsize] = _values[0]
+                elif operation == 'or':
+                    self._sparse_map[oldsize:][_pix[out_cov] + self._cov_map[ipnest_cov[out_cov]] -
+                                               oldsize] |= _values[0]
+                elif operation == 'and':
+                    self._sparse_map[oldsize:][_pix[out_cov] + self._cov_map[ipnest_cov[out_cov]] -
+                                               oldsize] &= _values[0]
             else:
-                sparse_append = np.zeros(new_cov_pix.size*self._cov_map.nfine_per_cov,
-                                         dtype=self._sparse_map.dtype)
-            # Fill with the empty defaults (generally UNSEEN)
-            sparse_append[:] = self._sparse_map[0]
-
-            # Update cov_index_map
-            # These are pixels that are at the end of the previous sparsemap
-
-            new_cov_map = self._cov_map.append_pixels(len(self._sparse_map), new_cov_pix, check=False)
-
-            # Fill in the pixels to append
-            if single_value:
-                sparse_append[_pix[out_cov] + new_cov_map[ipnest_cov[out_cov]] -
-                              len(self._sparse_map)] = values[0]
-            else:
-                sparse_append[_pix[out_cov] + new_cov_map[ipnest_cov[out_cov]] -
-                              len(self._sparse_map)] = values[out_cov]
-
-            # And set the values in the map
-            self._cov_map = new_cov_map
-            if self._is_wide_mask:
-                self._sparse_map = np.reshape(np.append(self._sparse_map,
-                                                        sparse_append),
-                                              (len(self._sparse_map) +
-                                               len(sparse_append),
-                                               self._wide_mask_width))
-            else:
-                self._sparse_map = np.append(self._sparse_map, sparse_append)
+                if operation == 'replace':
+                    self._sparse_map[oldsize:][_pix[out_cov] + self._cov_map[ipnest_cov[out_cov]] -
+                                               oldsize] = _values[out_cov]
+                elif operation == 'or':
+                    self._sparse_map[oldsize:][_pix[out_cov] + self._cov_map[ipnest_cov[out_cov]] -
+                                               oldsize] |= _values[out_cov]
+                elif operation == 'and':
+                    self._sparse_map[oldsize:][_pix[out_cov] + self._cov_map[ipnest_cov[out_cov]] -
+                                               oldsize] &= _values[out_cov]
 
     def set_bits_pix(self, pixels, bits, nest=True):
         """
@@ -578,12 +662,13 @@ class HealSparseMap(object):
         if np.max(bits) >= self._wide_mask_maxbits:
             raise ValueError("Bit position %d too large (>= %d)" % (np.max(bits),
                                                                     self._wide_mask_maxbits))
-        values = self.get_values_pix(pixels, nest=nest)
+
+        value = self._sparse_map[0].copy()
         for bit in bits:
             field, bitval = _get_field_and_bitval(bit)
-            values[:, field] |= bitval
+            value[field] |= bitval
 
-        self.update_values_pix(pixels, values, nest=nest)
+        self.update_values_pix(pixels, value, nest=nest, operation='or')
 
     def clear_bits_pix(self, pixels, bits, nest=True):
         """
@@ -602,12 +687,13 @@ class HealSparseMap(object):
         if np.max(bits) >= self._wide_mask_maxbits:
             raise ValueError("Bit position %d too large (>= %d)" % (np.max(bits),
                                                                     self._wide_mask_maxbits))
-        values = self.get_values_pix(pixels, nest=nest)
+
+        value = self._sparse_map[0].copy()
         for bit in bits:
             field, bitval = _get_field_and_bitval(bit)
-            values[:, field] &= ~bitval
+            value[field] |= ~bitval
 
-        self.update_values_pix(pixels, values, nest=nest)
+        self.update_values_pix(pixels, value, nest=nest, operation='and')
 
     def get_values_pos(self, theta_or_ra, phi_or_dec, lonlat=False, valid_mask=False):
         """
@@ -617,7 +703,9 @@ class HealSparseMap(object):
 
         Parameters
         ----------
-        theta_or_ra, phi_or_dec : float, array-like
+        theta_or_ra : `float`, array-like
+           Angular coordinates of points on a sphere.
+        phi_or_dec : `float`, array-like
            Angular coordinates of points on a sphere.
         lonlat : `bool`, optional
            If True, input angles are longitude and latitude in degrees.
@@ -640,7 +728,7 @@ class HealSparseMap(object):
 
         Parameters
         ----------
-        pixel : `np.array`
+        pixel : `np.ndarray`
            Integer array of healpix pixels.
         nest : `bool`, optional
            Are the pixels in nest scheme?  Default is True.
@@ -649,7 +737,7 @@ class HealSparseMap(object):
 
         Returns
         -------
-        values : `np.array`
+        values : `np.ndarray`
            Array of values/validity from the map.
         """
         if not nest:
@@ -683,7 +771,9 @@ class HealSparseMap(object):
 
         Parameters
         ----------
-        theta_or_ra, phi_or_dec : float, array-like
+        theta_or_ra : `float`, array-like
+           Angular coordinates of points on a sphere.
+        phi_or_dec : `float`, array-like
            Angular coordinates of points on a sphere.
         lonlat : `bool`, optional
            If True, input angles are longitude and latitude in degrees.
@@ -708,7 +798,7 @@ class HealSparseMap(object):
 
         Parameters
         ----------
-        pixel : `np.array`
+        pixel : `np.ndarray`
            Integer array of healpix pixels.
         nest : `bool`, optional
            Are the pixels in nest scheme?  Default is True.
@@ -747,7 +837,7 @@ class HealSparseMap(object):
 
         Returns
         -------
-        cov_map : `np.array`
+        cov_map : `np.ndarray`
            Float array of fractional coverage of each pixel
         """
 
@@ -934,8 +1024,8 @@ class HealSparseMap(object):
 
         if nside is specified, then reduce to that nside
 
-        Args:
-        -----
+        Parameters
+        ----------
         nside : `int`
             Output nside resolution parameter (should be a multiple of 2). If
             not specified the output resolution will be equal to the parent's
@@ -948,9 +1038,9 @@ class HealSparseMap(object):
             If the parent HealSparseMap contains recarrays, key selects the
             field that will be transformed into a HEALPix map.
 
-        Returns :
-        --------
-        hp_map : `np.array`
+        Returns
+        -------
+        hp_map : `np.ndarray`
             Output HEALPix map with the requested resolution.
         """
         # If no nside is passed, we generate a map with the same resolution as the original
@@ -984,7 +1074,7 @@ class HealSparseMap(object):
             dtypeOut = single_map._sparse_map.dtype
 
         # Create an empty HEALPix map, filled with UNSEEN values
-        hp_map = np.zeros(hp.nside2npix(nside), dtype=dtypeOut) + hp.UNSEEN
+        hp_map = np.full(hp.nside2npix(nside), hp.UNSEEN, dtype=dtypeOut)
 
         valid_pixels = single_map.valid_pixels
         hp_map[valid_pixels] = single_map.get_values_pix(valid_pixels)
@@ -998,24 +1088,16 @@ class HealSparseMap(object):
 
         Returns
         -------
-        valid_pixels : `np.array`
+        valid_pixels : `np.ndarray`
         """
+        if self._is_rec_array:
+            valid_pixel_inds, = np.where(self._sparse_map[self._primary] > self._sentinel)
+        elif self._is_wide_mask:
+            valid_pixel_inds, = np.where(self._sparse_map.sum(axis=1, dtype=np.bool))
+        else:
+            valid_pixel_inds, = np.where(self._sparse_map > self._sentinel)
 
-        # Get the coarse pixels that are in the map
-        valid_coverage, = np.where(self.coverage_mask)
-        nfine_per_cov = 2**self._cov_map.bit_shift
-
-        # For each coarse pixel, this is the starting point for the pixel number
-        pixBase = np.left_shift(valid_coverage, self._cov_map.bit_shift)
-
-        # Tile/repeat to expand into the full pixel numbers
-        # Note that these are all the pixels that are defined in the sparse map,
-        # but not all of them are valid
-        valid_pixels = (np.tile(np.arange(nfine_per_cov), valid_coverage.size) +
-                        np.repeat(pixBase, nfine_per_cov))
-
-        # And return only the valid subset
-        return valid_pixels[self.get_values_pix(valid_pixels, valid_mask=True)]
+        return valid_pixel_inds - self._cov_map[self._cov_map.cov_pixels_from_index(valid_pixel_inds)]
 
     def valid_pixels_pos(self, lonlat=False, return_pixels=False):
         """
@@ -1069,7 +1151,7 @@ class HealSparseMap(object):
         # Count the number of filled pixels in the coverage mask
         npop_pix = np.count_nonzero(self.coverage_mask)
         # We need the new bit_shifts and we have to build a new CovIndexMap
-        bit_shift = 2 * int(np.round(np.log(nside_out / self.nside_coverage) / np.log(2)))
+        bit_shift = _compute_bitshift(self.nside_coverage, nside_out)
         nfine_per_cov = 2**bit_shift
         # Work with RecArray (we have to change the resolution to all maps...)
         if self._is_rec_array:
@@ -1233,7 +1315,7 @@ class HealSparseMap(object):
         """
         if isinstance(key, int):
             # Set a single pixel
-            return self.update_values_pix(np.array([key]), np.array([value]))
+            return self.update_values_pix(np.array([key]), value)
         elif isinstance(key, slice):
             # Set a slice of pixels
             start = key.start if key.start is not None else 0
@@ -1270,19 +1352,25 @@ class HealSparseMap(object):
         if not self._is_rec_array:
             raise TypeError("HealSparseMap is not a recarray map")
 
-        # This will not copy memory, which allows in-recarray assignment
-        # Problems can happen with mixed type recarrays, unfortunately.
-        # However, undefined behavior will happen with in-place operations
-        # over invalid pixels no matter what.
+        # If we are the primary key, use the sentinel as set.  Otherwise,
+        # use the default sentinel unless otherwise overridden.
+        if key == self._primary:
+            _sentinel = check_sentinel(self._sparse_map[key].dtype.type, self._sentinel)
+        else:
+            _sentinel = check_sentinel(self._sparse_map[key].dtype.type, sentinel)
+
         if not copy:
-            # This is easy, and no replacements need to happen
+            # This will not copy memory which allows in-recarray assignment.
+            # Problems can potentially happen with mixed type recarrays depending
+            # on how they were constructed (though using make_empty should be safe).
+            # However, these linked maps cannot be used to add new pixels which
+            # is why there is the _is_view flag.
             return HealSparseMap(cov_map=self._cov_map,
                                  sparse_map=self._sparse_map[key],
-                                 nside_sparse=self._nside_sparse, sentinel=self._sentinel)
+                                 nside_sparse=self._nside_sparse, sentinel=_sentinel,
+                                 _is_view=True)
 
-        _sentinel = check_sentinel(self._sparse_map[key].dtype.type, sentinel)
-
-        new_sparse_map = np.zeros_like(self._sparse_map[key]) + _sentinel
+        new_sparse_map = np.full_like(self._sparse_map[key], _sentinel)
 
         valid_indices = (self._sparse_map[self._primary] > self._sentinel)
         new_sparse_map[valid_indices] = self._sparse_map[key][valid_indices]
